/*  ISELEDuino FastLED Sample program.
 *  with this modified FastLED Library sketches can be easily re-used.
 *  However, please do this in an own directory, because you have to delete here your original FastLED Lib.
 *  It is recommended to do manual installs, because installing libraries over the Library Manager places the  
 *  libs in areas like C:\User\<username>\AppData\Roaming\Arduino .....
 *  
 *  Please set the amount of LEDs directly in ISELED.h, line 17
 *  Please set the LED Brand in ISELED.h, line 27
 *  
 *  We have to choose Arduino MKR 1000 as board (MKR Zero is not supported by this FastLED version (3.4) 
 *  
 *  Hebatronic & BroogleLabs
 *  July 2023
 */

#include <BOARD.h>
#include <drv_iseled_defines.h>
#include <ISELED.h>
#include <FastLED.h>

//#define LED_TYPE    ISELED_LED
#define LED_TYPE    WS2812
#define COLOR_ORDER RGB
#define NUM_LEDS    NumberOfLEDs
#define DIM_LEVEL   3                          // 0 = brightest 3 = dimmest
#define BRIGHTNESS  32                         // watch the power! one ISELED draws typically 32mA at white and full brightness
                                               // USB Power is typically 500mA; 100mA for the ISELEDuino; leaves current for 12 LEDs
                                               // BRIGHTNESS 64 should be good for max 50 LEDs
#define FRAMES_PER_SECOND  120                 // because we can!
struct CRGB leds[NUM_LEDS];                    // FastLED specific struct

void FastISELED_show(){                        // magic helper function for ISELED
  uint16_t i_fz; 
   for (i_fz = 0; i_fz < NUM_LEDS; i_fz++) { 
     digLED_Set_RGB(leds[i_fz].red,leds[i_fz].green,leds[i_fz].blue, i_fz+START_ADDRESS, 0);
  }
}

void setup() {
  delay(1000);                        // 1 second delay for recovery
  // tell FastLED about the LED strip configuration
  FastLED.addLeds<LED_TYPE,COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  FastLED.setBrightness(BRIGHTNESS);  // set master brightness control
  HARDWARE_Initialize();              // Initialize the SAM hardware modules and pin's 
  delayMicroseconds(200);             // wait ISELED POR time 
  digLED_start();                     // Initialize the ISELED string and test the communication 
  digLED_Set_DIM(DIM_LEVEL,DIM_LEVEL,DIM_LEVEL,0,0);
}

// this is FastLED's famous DemoReel, written by Marc Kriegsman.

// List of patterns to cycle through.  Each is defined as a separate function below.
typedef void (*SimplePatternList[])();
SimplePatternList gPatterns = { blink_red, fade_in_out, move_brightness, rainbow, rainbowWithGlitter, confetti, /*sinelon,*/ juggle, bpm };

uint8_t gCurrentPatternNumber = 0; // Index number of which pattern is current
uint8_t gHue = 0; // rotating "base color" used by many of the patterns
  
void loop()
{
  // Call the current pattern function once, updating the 'leds' array
  gPatterns[gCurrentPatternNumber]();
  
//  FastLED.show();  // send the 'leds' array out to the actual LED strip
  FastISELED_show(); // simply replace FastLED.show(); with FastISELED_show();
  
  FastLED.delay(1000/FRAMES_PER_SECOND);   // insert a delay to keep the framerate modest

  // do some periodic updates
  EVERY_N_MILLISECONDS( 20 ) { gHue++; }   // slowly cycle the "base color" through the rainbow
  EVERY_N_SECONDS( 10 ) { nextPattern(); } // change patterns periodically
}

#define ARRAY_SIZE(A) (sizeof(A) / sizeof((A)[0]))

void nextPattern()
{
  // add one to the current pattern number, and wrap around at the end
  gCurrentPatternNumber = (gCurrentPatternNumber + 1) % ARRAY_SIZE( gPatterns);
}

// ISELED basics ------------------------------------------------------------------------------

void blink_red() {              // this is the absolutely basic ISELED programming
  digLED_Set_DIM(3,3,3,0,0);    // set DIM level {0..3] (0 = brightest 3 = dimmest
  digLED_Set_RGB(64,0,0,0,0);   // set colors (r,g,b,address,branch) 
  delay(500);                   // r,g,b: 0..255; address 1..4079 (0=all), branch not used yet (=0)
  digLED_Set_RGB(0,0,0,0,0);    // switch all off
  delay(500);  
}

void fade_in_out() {
  digLED_Set_DIM(3,3,3,0,0);    // set DIM level {0..3] (0 = brightest 3 = dimmest
  digLED_Set_RGB(0,0,0,0,0);    // switch all off
  for (int i = 0; i<=255; i++) {   // fade in
  digLED_Set_RGB(0, i, i, 0, 0); 
  delay(20);                    // 1000ms / 20ms = 50 frames per second 
  }
  for (int i = 255; i>=0; i--) {   // fade out
  digLED_Set_RGB(0, i, i, 0, 0); 
  delay(20); }
}

void move_brightness() {
  digLED_Set_DIM(3,3,3,0,0);    // set DIM level {0..3] (0 = brightest 3 = dimmest
  digLED_Set_RGB(0,0,0,0,0);    // switch all off
  for (int i = 1; i <= NUM_LEDS; i++) {
  digLED_Set_RGB(i,0,i*4,i,0);  // how many LEDs did you declare in ISELED.h, line 17
  delay(200);  
  }
}
// end ISELED basics --------------------------------------------------------------------------

// FastLED fancy functions --------------------------------------------------------------------

void rainbow() 
{
  // FastLED's built-in rainbow generator
  fill_rainbow( leds, NUM_LEDS, gHue, 7);
}

void rainbowWithGlitter() 
{
  // built-in FastLED rainbow, plus some random sparkly glitter
  rainbow();
  addGlitter(80);
}

void addGlitter( fract8 chanceOfGlitter) 
{
  if( random8() < chanceOfGlitter) {
    leds[ random16(NUM_LEDS) ] += CRGB::White;
  }
}

void confetti() 
{
  // random colored speckles that blink in and fade smoothly
  fadeToBlackBy( leds, NUM_LEDS, 10);
  int pos = random16(NUM_LEDS);
  leds[pos] += CHSV( gHue + random8(64), 200, 255);
}

void sinelon()
{
  // a colored dot sweeping back and forth, with fading trails
  fadeToBlackBy( leds, NUM_LEDS, 20);
  int pos = beatsin16( 13, 0, NUM_LEDS-1 );
  leds[pos] += CHSV( gHue, 255, 192);
}

void bpm()
{
  // colored stripes pulsing at a defined Beats-Per-Minute (BPM)
  uint8_t BeatsPerMinute = 62;
  CRGBPalette16 palette = PartyColors_p;
  uint8_t beat = beatsin8( BeatsPerMinute, 64, 255);
  for( int i = 0; i < NUM_LEDS; i++) { //9948
    leds[i] = ColorFromPalette(palette, gHue+(i*2), beat-gHue+(i*10));
  }
}

void juggle() {
  // eight colored dots, weaving in and out of sync with each other
  fadeToBlackBy( leds, NUM_LEDS, 20);
  byte dothue = 0;
  for( int i = 0; i < 8; i++) {
    leds[beatsin16( i+7, 0, NUM_LEDS-1 )] |= CHSV(dothue, 200, 255);
    dothue += 32;
  }
}


/*
modifications for ISELED

#include <BOARD.h>     // for SAMD21
//#include <BOARD_E51.h>   // for SAME51
#include <drv_iseled_defines.h>
#include <ISELED.h>
#include <FastLED.h>

#define LED_TYPE    WS2812
#define COLOR_ORDER RGB                       // this is different to the actual WS2812 (they are GRB)
#define NUM_LEDS    NumberOfLEDs
#define DIM_LEVEL   3                         // watch the power!
#define BRIGHTNESS 250                        // watch the power!

struct CRGB leds[NUM_LEDS];

// this is the helper function to convert FastLED to ISELED
void setup() {
  delay(100);
  // tell FastLED about the LED strip configuration
  FastLED.addLeds<LED_TYPE,COLOR_ORDER>(leds, NUM_LEDS);
  FastLED.setBrightness(BRIGHTNESS);
  
HARDWARE_Initialize();  // Initialize the SAM hardware modules and pin's 
delayMicroseconds(200); // wait ISELED POR time 
  // Initialize the ISELED string and test the communication 
  digLED_start();
  digLED_Set_DIM(DIM_LEVEL,DIM_LEVEL,DIM_LEVEL,0,0);
}

//  FastLED.show();  // send the 'leds' array out to the actual LED strip
  FastISELED_show(); // simply replace FastLED.show(); with FastISELED_show();

 */
